#property copyright   "2016, Elia Burstein."
#property link        "http://www.mql4.com"
#property description "Super Smooth Moving Averages Convergence/Divergence"
#property strict

#include <superMATools.mqh>

//--- indicator settings
#property  indicator_separate_window
#property  indicator_buffers 3
#property  indicator_color1  clrLime
#property  indicator_color2  clrRed
#property  indicator_color3  clrSilver
//--- indicator parameters
input double InpFastMA=12.0;   // Fast SSMA Period
input double InpSlowMA=26.0;   // Slow SSMA Period
input double InpSignalMA=9.0;  // Signal SSMA Period
//--- indicator buffers
double   macdBuffer[],ExtMacdBullishBuffer[],ExtMacdBearishBuffer[];
double   ExtSignalBuffer[];
double   fastMABuffer[],slowMABuffer[];
double   attr1[3],attr2[3],attr3[3];

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit(void)
  {
   IndicatorDigits(Digits+1);
//--- drawing settings
   SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexStyle(1,DRAW_HISTOGRAM);
   SetIndexStyle(2,DRAW_LINE);
   SetIndexDrawBegin(0,BARS_USED-1);
   SetIndexDrawBegin(1,BARS_USED-1);
   SetIndexDrawBegin(2,BARS_USED-1);
//--- indicator buffers mapping
   IndicatorBuffers(6);
   SetIndexBuffer(0,ExtMacdBullishBuffer,INDICATOR_DATA);
   SetIndexBuffer(1,ExtMacdBearishBuffer,INDICATOR_DATA);
   SetIndexBuffer(2,ExtSignalBuffer,INDICATOR_DATA);
   SetIndexBuffer(3,macdBuffer,INDICATOR_CALCULATIONS);
   SetIndexBuffer(4,fastMABuffer,INDICATOR_CALCULATIONS);
   SetIndexBuffer(5,slowMABuffer,INDICATOR_CALCULATIONS);
   rescale();
   ArraySetAsSeries(ExtMacdBullishBuffer,true);
   ArraySetAsSeries(ExtMacdBearishBuffer,true);
   ArraySetAsSeries(ExtSignalBuffer,true);
   ArraySetAsSeries(macdBuffer,true);
   ArraySetAsSeries(fastMABuffer,true);
   ArraySetAsSeries(slowMABuffer,true);
//--- name for DataWindow and indicator subwindow label
   IndicatorShortName("SSMACD("+DoubleToString(InpFastMA,1)+","+DoubleToString(InpSlowMA,1)+","+DoubleToString(InpSignalMA,1)+")");
   SetIndexLabel(0,"MACDBull");
   SetIndexLabel(1,"MACDBear");
   SetIndexLabel(2,"Signal");

   if(InpFastMA<=1 || InpSlowMA<=1 || InpSignalMA<=1 || InpFastMA>=InpSlowMA)
   {
      Print("Wrong input parameters");
      return(INIT_PARAMETERS_INCORRECT);
   }
   attenuation(InpFastMA*2,attr1);
   attenuation(InpSlowMA*2,attr2);
   attenuation(InpSignalMA*2,attr3);
//--- initialization done
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Moving Averages Convergence/Divergence                           |
//+------------------------------------------------------------------+
int OnCalculate (const int rates_total,
                 const int prev_calculated,
                 const datetime& time[],
                 const double& open[],
                 const double& high[],
                 const double& low[],
                 const double& close[],
                 const long& tick_volume[],
                 const long& volume[],
                 const int& spread[])
{
   ArraySetAsSeries(close,true);

   if(rates_total<BARS_USED) return(0);
//--- last counted bar will be recounted
   if (prev_calculated<0) return(-1);
   int i,limit = rates_total-((prev_calculated==0)?1:prev_calculated-1);

   for(i=limit; i>=0 && !IsStopped(); i--)
   {
      if(i>rates_total-BARS_USED)
      {
         fastMABuffer[i]=slowMABuffer[i]=close[i];
         macdBuffer[i]=0;
      }
      else
      {
         superSmooth(i,PRICE_CLOSE,attr1,fastMABuffer);
         superSmooth(i,PRICE_CLOSE,attr2,slowMABuffer);
         macdBuffer[i]=fastMABuffer[i]-slowMABuffer[i];
         if(macdBuffer[i]>macdBuffer[i+1] || (macdBuffer[i]==macdBuffer[i+1] && ExtMacdBullishBuffer[i+1]==macdBuffer[i+1]))
         {
            ExtMacdBullishBuffer[i]=macdBuffer[i];
            ExtMacdBearishBuffer[i]=EMPTY_VALUE;
         }
         else
         {
            ExtMacdBearishBuffer[i]=macdBuffer[i];
            ExtMacdBullishBuffer[i]=EMPTY_VALUE;
         }
//--- signal line counted in the 2-nd buffer
         if(i>rates_total-BARS_USED*2+1)
            ExtSignalBuffer[i]=macdBuffer[i];
         else
            superSmooth2Phase2(i,attr3,macdBuffer,ExtSignalBuffer);
      }
   }

   return(rates_total);
  }

void rescale()
{
   static int oldScale=0;
   int scale=(int)ChartGetInteger(0,CHART_SCALE,0);
   if(scale!=oldScale)
   {
      oldScale=scale;
      int thickness=(int)(13/MathPow(2,5-scale));
      if(thickness<2)thickness=2;
      SetIndexStyle(0,DRAW_HISTOGRAM,STYLE_SOLID,thickness);
      SetIndexStyle(1,DRAW_HISTOGRAM,STYLE_SOLID,thickness);
   }
}

void OnChartEvent(const int id,         // Event ID
                  const long& lparam,   // Parameter of type long event
                  const double& dparam, // Parameter of type double event
                  const string& sparam  // Parameter of type string events
)
{
   if(id==CHARTEVENT_CHART_CHANGE)
   {
      rescale();
      WindowRedraw();
   }
   
}
