#property copyright   "2016, Elia Burstein."
#property link        "http://www.mql4.com"
#property description "Super Smooth Moving Averages Convergence/Divergence"
#property strict

#include <superMATools.mqh>

//--- indicator settings
#property  indicator_separate_window
#property  indicator_buffers 2
#property  indicator_color1  Silver
#property  indicator_color2  Red
#property  indicator_width1  2
//--- indicator parameters
input double InpFastMA=12.0;   // Fast SSMA Period
input double InpSlowMA=26.0;   // Slow SSMA Period
input double InpSignalMA=9.0;  // Signal SSMA Period
//--- indicator buffers
double   ExtMacdBuffer[];
double   ExtSignalBuffer[];
double   fastMABuffer[],slowMABuffer[];
double   attr1[3],attr2[3],attr3[3];

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit(void)
  {
   IndicatorDigits(Digits+1);
//--- drawing settings
   SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexStyle(1,DRAW_LINE);
   SetIndexDrawBegin(0,BARS_USED-1);
   SetIndexDrawBegin(1,BARS_USED-1);
//--- indicator buffers mapping
   IndicatorBuffers(4);
   SetIndexBuffer(0,ExtMacdBuffer,INDICATOR_DATA);
   SetIndexBuffer(1,ExtSignalBuffer,INDICATOR_DATA);
   SetIndexBuffer(2,fastMABuffer,INDICATOR_CALCULATIONS);
   SetIndexBuffer(3,slowMABuffer,INDICATOR_CALCULATIONS);
   ArraySetAsSeries(ExtMacdBuffer,true);
   ArraySetAsSeries(ExtSignalBuffer,true);
   ArraySetAsSeries(fastMABuffer,true);
   ArraySetAsSeries(slowMABuffer,true);
//--- name for DataWindow and indicator subwindow label
   IndicatorShortName("SSMACD("+DoubleToString(InpFastMA,1)+","+DoubleToString(InpSlowMA,1)+","+DoubleToString(InpSignalMA,1)+")");
   SetIndexLabel(0,"MACD");
   SetIndexLabel(1,"Signal");

   if(InpFastMA<=1 || InpSlowMA<=1 || InpSignalMA<=1 || InpFastMA>=InpSlowMA)
   {
      Print("Wrong input parameters");
      return(INIT_PARAMETERS_INCORRECT);
   }
   attenuation(InpFastMA*2,attr1);
   attenuation(InpSlowMA*2,attr2);
   attenuation(InpSignalMA*2,attr3);
//--- initialization done
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Moving Averages Convergence/Divergence                           |
//+------------------------------------------------------------------+
int OnCalculate (const int rates_total,
                 const int prev_calculated,
                 const datetime& time[],
                 const double& open[],
                 const double& high[],
                 const double& low[],
                 const double& close[],
                 const long& tick_volume[],
                 const long& volume[],
                 const int& spread[])
{
   ArraySetAsSeries(close,true);

   if(rates_total<BARS_USED) return(0);
//--- last counted bar will be recounted
   if (prev_calculated<0) return(-1);
   int i,limit = rates_total-((prev_calculated==0)?1:prev_calculated-1);

   for(i=limit; i>=0 && !IsStopped(); i--)
   {
      if(i>rates_total-BARS_USED)
         fastMABuffer[i]=slowMABuffer[i]=close[i];
      else
      {
         superSmooth(i,PRICE_CLOSE,attr1,fastMABuffer);
         superSmooth(i,PRICE_CLOSE,attr2,slowMABuffer);
         ExtMacdBuffer[i]=fastMABuffer[i]-slowMABuffer[i];
//--- signal line counted in the 2-nd buffer
         if(i>rates_total-BARS_USED*2+1)
            ExtSignalBuffer[i]=ExtMacdBuffer[i];
         else
            superSmooth2Phase2(i,attr3,ExtMacdBuffer,ExtSignalBuffer);
      }
   }

   return(rates_total);
  }
//+------------------------------------------------------------------+