#property copyright "Copyright  2016, Elia Burstein"
#property link      "https://www.mql5.com"
#property description "Description:"
#property description "SuperSmoother = 2-Pole Modified Butterworth Filter applied to SMA(2)"
#property description "Recommended for universal use to remove aliasing noise"
#property description "Best used with AttenuationPeriod = 10"
#property description "Use higher AttenuationPeriod if necessary, lower is not advised"
#property description "Feed output data into other indicators for analysis if required"
#property description "Better than using raw price data due to aliasing noise"
#property strict

#include <superMATools.mqh>

#property indicator_chart_window
#property indicator_buffers 2
#property indicator_color1 clrLime
#property indicator_color2 clrRed
#property indicator_type1 DRAW_LINE
#property indicator_type2 DRAW_LINE
#property indicator_style1 STYLE_SOLID
#property indicator_style2 STYLE_SOLID
#property indicator_width1 3
#property indicator_width2 3
//---- indicator parameters
input double               HAP    = 5;             // Half Attenuation Period
input ENUM_APPLIED_PRICE   Apply2 = PRICE_CLOSE;   // Apply to
input int                  Shift  = 0;             // Shift
//---- indicator buffers
double ExtMapBufferFull[];
double ExtMapBuffer1[];
double ExtMapBuffer2[];
//----

double attr[3];

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit(){  
//---- drawing settings
   SetIndexShift(0,-1*Shift);
   SetIndexShift(1,-1*Shift);
   IndicatorDigits(Digits());
   SetIndexDrawBegin(0,BARS_USED-1);
   SetIndexDrawBegin(1,BARS_USED-1);
   if(HAP<1.0) 
   {
      Print("Please select higher period");
      return(INIT_PARAMETERS_INCORRECT);
   }
//---- indicator short name
   string Sapply2[];
   StringSplit(EnumToString(Apply2),StringGetCharacter("_",0),Sapply2);
   IndicatorShortName("SuperSmootherV2("+DoubleToString(HAP,1)+","+Sapply2[1]+")");
//---- indicator buffers mapping
   IndicatorBuffers(3);
   SetIndexBuffer(0,ExtMapBuffer1,INDICATOR_DATA);
   SetIndexBuffer(1,ExtMapBuffer2,INDICATOR_DATA);
   SetIndexBuffer(2,ExtMapBufferFull,INDICATOR_CALCULATIONS);
   ArraySetAsSeries(ExtMapBuffer1,true);
   ArraySetAsSeries(ExtMapBuffer2,true);
   ArraySetAsSeries(ExtMapBufferFull,true);
   SetIndexLabel(0,"Bull");
   SetIndexLabel(1,"Bear");
//   SetIndexLabel(1,"Bear");
//---- initialize constants
   attenuation(HAP*2,attr);
//---- initialization done
   return(INIT_SUCCEEDED);
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,      // size of input time series 
                 const int prev_calculated,  // bars handled in previous call 
                 const datetime& time[],     // Time 
                 const double& open[],       // Open 
                 const double& high[],       // High 
                 const double& low[],        // Low 
                 const double& close[],      // Close 
                 const long& tick_volume[],  // Tick Volume 
                 const long& volume[],       // Real Volume 
                 const int& spread[]         // Spread 
){
   if(rates_total<BARS_USED) return(0);
   if (prev_calculated<0) return(-1);
   int i,pos = rates_total-((prev_calculated==0)?1:prev_calculated-1);
   for(i=pos;i>=0;i--)
   {
      if(i>rates_total-BARS_USED)
         ExtMapBufferFull[i]=usePrice(i,Apply2);
      else
      {
         superSmooth(i,Apply2,attr,ExtMapBufferFull);
         if(ExtMapBufferFull[i]>ExtMapBufferFull[i+1])
            drawBull(i);
         else
            drawBear(i);
      }
   }
   return(rates_total);
}

void drawBull(int pos) {
   ExtMapBuffer1[pos]=ExtMapBufferFull[pos];
   ExtMapBuffer1[pos+1]=ExtMapBufferFull[pos+1];
   ExtMapBuffer2[pos]=EMPTY_VALUE;
}
void drawBear(int pos) {
   ExtMapBuffer2[pos]=ExtMapBufferFull[pos];
   ExtMapBuffer2[pos+1]=ExtMapBufferFull[pos+1];
   ExtMapBuffer1[pos]=EMPTY_VALUE;                           
}